/*
 *
 * Description:		Main part of the server for the file upload system
 *
 */

/*
 * To compile:
 *
 *  Set up the following
 *
 *   - Platform SDK
 *   - MySQL development files
 *   - MySQL++
 *   - Winsock2
 *
 *  Set directory paths to point to the respective include and library files
 *
 */

#include <iostream>
#include "cmsusers.h"
#include "cmsdbfunc.h"
#include "cmsserverclass.h"
#include "cmscommandqueue.h"

using namespace std;

int main (int argc, char *argv[]){
	/***************************************************************
	 *    Present a welcome screen when starting server
	 ***************************************************************/

	cout << "Content Management Server v.01alpha\n\n";

	/***************************************************************
	 *    Test the number of aguments used to call this application
	 ***************************************************************/

	if (argc != 7){
		cout << "cms.exe <dbname> <dbipaddress> <dbuser> <passwd> <basefilepath> <iplisten>\n";
		return 0;
	}

	/***************************************************************
	 *    Make server listen for connections
	 ***************************************************************/
	//TODO: Check for errors, put inside while loop, try to restore
	//      listening state in error conditions, make a "state-machine"
	int received = 0;
	char *recvbuf = new char[1026];
	char *tmpstr = 0;
	cmsserverclass *cmssc = new cmsserverclass(2000);

	if (cmssc->lerror() != 0){
		cout << "Error starting server subsystem\n";
		return 0;
	}

	if (cmssc->connect(argv[6]) != 0){
		cout << "Error starting server subsystem\n";
		return 0;
	}

	//Create a list of connected users
	cmsusers *cmsu = new cmsusers();
	cmsu->setbasepath(argv[5]);

	//The server loop
	int socket_id = 0;
	bool action = false;
	while (1){	//Make so that an admin can shutdown remotely

		/***************************************************************
		 *    Check for new connections, remove finished connections,
		 *        update the table of connected users
		 ***************************************************************/

		socket_id = cmssc->checkforconnection();
		if (socket_id != 0){
			if (!cmsu->userexists(socket_id)){
				action = true;
				cout << "ID: " << socket_id << ", connected\n";
				cmsu->adduser(socket_id);
			}
		}

		/***************************************************************
		 *    Fetch data from users if there are data waiting in buffers,
		 *        respond correspondingly
		 ***************************************************************/

		received = 1025;
		socket_id = cmssc->readConnection(recvbuf, &received);

		if (received == -1){
			//Tell system that file is successfully uploaded
			//Bug: Might actually not be the case (some other error)
			//This will cause the file to be wrongly tagged as a finished upload
			if (!cmsu->getclosed(socket_id)){
				cmsu->setclosed(socket_id);

				//Connect to the database
				cmsdbfunc *dbf = new cmsdbfunc();

				dbf->setdb(argv[1]);
				dbf->setdbhost(argv[2]);
				dbf->setdbuser(argv[3]);
				dbf->setdbpasswd(argv[4]);

				dbf->setuploaded(cmsu->getfileid(socket_id));
			}

			//Prevent the system to handle this node further
			socket_id = 0;
			received = 0;
		}

		//trick to prevent buffer overflow and to detect correct sizes of buffers
		if (received<1025){
			*(recvbuf+received) = 0x00;
		} else {
			*(recvbuf+1026) = 0x00;
		}

		//There is communication pending from the current socket
		if (socket_id != 0){
			action = true;

			//Is entered username pending?
			int logonstep = cmsu->getlogonstep(socket_id);
			if (logonstep == 0){	//Logon name
				tmpstr = cmsu->createstring(recvbuf);
				cout << "ID: " << socket_id << ", user: " << tmpstr << "\n";
				cmsu->setname(socket_id, tmpstr);
				cmsu->inclogonstep(socket_id);
			}

			//What about password?
			else if (logonstep == 1){	//Logon password
				tmpstr = cmsu->createstring(recvbuf);
				cout << "ID: " << socket_id << ", password: (hidden)\n"; //<< tmpstr << "\n";
				cmsu->setpassword(socket_id, tmpstr);
				cmsu->inclogonstep(socket_id);

				//compare this user to the mysql user database
				cmsdbfunc *dbf = new cmsdbfunc();

				dbf->setdb(argv[1]);
				dbf->setdbhost(argv[2]);
				dbf->setdbuser(argv[3]);
				dbf->setdbpasswd(argv[4]);

				if (dbf->userlogin(cmsu->getname(socket_id), cmsu->getpassword(socket_id))){
					cout << "ID: " << socket_id << ", successfully validated against user database" << "\n";
					cmsu->setvalidated(socket_id, true);
					cmsu->setorgname(socket_id, dbf->getorgname());
					cmsu->setorgid(socket_id, dbf->getorgid());
				} else {
					cout << "ID: " << socket_id << ", could not validate against user database" << "\n";
					//TODO: Should force socket close here
				}
				delete dbf;
			}

			//The file name?
			else if (logonstep == 2){	//logon transfer filename
				tmpstr = cmsu->createstring(recvbuf);
				cout << "ID: " << socket_id << ", file name: " << tmpstr << "\n";
				cmsu->setfilename(socket_id, tmpstr);
				cmsu->inclogonstep(socket_id);
			}

			//File size?
			else if (logonstep == 3){	//logon transfer filename
				tmpstr = cmsu->createstring(recvbuf);
				cout << "ID: " << socket_id << ", file size: " << tmpstr << "\n";
				cmsu->inclogonstep(socket_id);

				//Connect to the database
				cmsdbfunc *dbf = new cmsdbfunc();

				dbf->setdb(argv[1]);
				dbf->setdbhost(argv[2]);
				dbf->setdbuser(argv[3]);
				dbf->setdbpasswd(argv[4]);

				//TODO: optimize all calls to same userobject
				cmsu->setfileid(socket_id, dbf->addfile(cmsu->getorgid(socket_id), cmsu->getdbfilename(socket_id)));

				delete dbf;
			}

			//Receive file from the current connection
			else if (logonstep != -1){	//logged on: do receive
				if (received > 0){
					action = true;
					//TODO: remove commented code? (For debug)
					//cout << "ID: " << socket_id << ", read count: " << received;
					//if (!cmsu->writefile(socket_id, recvbuf, &received)) cout << " (refused)";
					//cout << "\n";

					cmsu->writefile(socket_id, recvbuf, &received);
				}
			}
		}

		/***************************************************************
		 *    If no communication this turn, wait to relieve the system
		 ***************************************************************/
		if (!action) {Sleep(100);/* cout << ".";*/};
		action = false;
	}

	//Cleanup
	delete cmsu;

	//Exit program
	return 0;

}
