/***************************************************************
 * Class cmsusers:
 *    Manage currently connected users
 ***************************************************************/

#include "cmsusers.h"
//#include <iostream>

//using namespace std;

/***************************************************************
 * Constructor:
 *    Sets the initial state for a new instance of this class
 ***************************************************************/

cmsusers::cmsusers(){
	nextuserid = 1;
	userlist = 0;
	firstuser = 0;
	lastuser = 0;
	basepath = 0;
}

/***************************************************************
 * Destructor:
 *    Cleans up when destroying this class
 ***************************************************************/

cmsusers::~cmsusers(){

	//As long as there are users
	while (firstuser != 0){
		//Prepare the next user while the pointer is valid
		lastuser = firstuser->nextuser;

		//Does the current user have a set name?
		if (firstuser->name != 0){
			//Delete the users name
			delete firstuser->name;
		}
		//Delete the current user
		delete firstuser;

		//Process the next user on the next pass
		firstuser = lastuser;
	}
}

/***************************************************************
 ***************************************************************
 ** Private method implementations
 ***************************************************************
 ***************************************************************/


/***************************************************************
 * Method: finduser
 *    Searches a list of users for a given id, and
 *    returns the user object
 ***************************************************************/

LPUSER cmsusers::finduser(int id){
	//Start search from the first user in the list
	LPUSER tempuser = firstuser;
	
	//As long as there are more users to search
	while (tempuser != 0){
		//Check if the right users ID is found (and return the user object)
		if (tempuser->id == id) return tempuser;

		//No match: prepare for the next user to check
		tempuser = tempuser->nextuser;
	}

	//No user with that ID was found
	return 0;
}

bool cmsusers::userexists(int id){
	//Start search from the first user in the list
	LPUSER tempuser = firstuser;
	
	//As long as there are more users to search
	while (tempuser != 0){
		//Check if the right users ID is found (and return the user object)
		if (tempuser->id == id) return true;

		//No match: prepare for the next user to check
		tempuser = tempuser->nextuser;
	}

	//No user with that ID was found
	return false;
}


/***************************************************************
 * Method: getszlength
 *    Returns the length of the given zero terminated string
 *    including the zero itself
 ***************************************************************/

int cmsusers::getszlength(char *instring){
	//In case of null pointer input
	if (instring == 0) return 0;

	//Search from the beginnig of the string
	int count = 0;

	//Until first occurence of a null character
	char ch = 0x00;
	do{
		ch = *(instring+(count++));
	} while ((ch != '\0') && (ch != 0x0a));

	*((instring+(count))-1) = 0x00;

	//return the found count (plus one)
	return count+1;
}

/***************************************************************
 ***************************************************************
 ** Public method implementations
 ***************************************************************
 ***************************************************************/


/***************************************************************
 * Method: adduser
 *    Adds a user to the list of connected users
 *    returning an ID for the created object
 ***************************************************************/

int cmsusers::adduser(){
	//Create the new object
	LPUSER newuser = new USER;

	//Assign it initial values
	newuser->id = nextuserid++;
	newuser->name = 0;
	newuser->password = 0;
	newuser->savefilename = 0;
	newuser->filename = 0;
	newuser->orgname = 0;
	newuser->orgid = 0;
	newuser->fpos = -1;
	newuser->ofs = 0;
	newuser->filesize = 0;
	newuser->logonstep = 0;
	newuser->connected = false;
	newuser->validated = false;
	newuser->nextuser = 0;
	newuser->closed = false;

	//Add it to the list
	if (firstuser == 0){
		firstuser = newuser;
		lastuser = newuser;
	} else {
		lastuser->nextuser = newuser;
		newuser->nextuser = firstuser;
		lastuser = newuser;
	}

	//Return the ID
	return newuser->id;
}

int cmsusers::adduser(int id){
	//Create the new object
	LPUSER newuser = new USER;

	//Assign it initial values
	newuser->id = id;
	newuser->name = 0;
	newuser->password = 0;
	newuser->savefilename = 0;
	newuser->filename = 0;
	newuser->orgname = 0;
	newuser->orgid = 0;
	newuser->fpos = -1;
	newuser->ofs = 0;
	newuser->filesize = 0;
	newuser->logonstep = 0;
	newuser->connected = false;
	newuser->validated = false;
	newuser->nextuser = 0;
	newuser->closed = false;

	//Add it to the list
	if (firstuser == 0){
		firstuser = newuser;
		lastuser = newuser;
	} else {
		lastuser->nextuser = newuser;
		newuser->nextuser = firstuser;
		lastuser = newuser;
	}

	//Return the ID
	return newuser->id;
}

void cmsusers::removeuser(int id){
	if (firstuser == 0) return;

	if (firstuser->id == id){
		if (firstuser->nextuser == firstuser){
			delete firstuser;
			firstuser = 0;
			lastuser = 0;
			return;
		}

		LPUSER tempuser = firstuser;
		firstuser = firstuser->nextuser;
		lastuser->nextuser = firstuser;
		delete tempuser;

		return;
	}

	LPUSER tempuser = firstuser;
	while (tempuser->nextuser->id != id){
		tempuser = tempuser->nextuser;
	}

	if (tempuser->nextuser == lastuser){
		delete lastuser;
		lastuser = tempuser->nextuser;
		tempuser->nextuser = firstuser;
	}


}

/***************************************************************
 * Method: setname
 *    Names a user by his or her ID
 *    returning the ID for the user modified,
 *    or 0 if the user was not found
 ***************************************************************/

int cmsusers::setname(int id, char *name){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){

		//Is the user already named?
		if (tempuser->name != 0){
			//Remove the old name
			delete tempuser->name;
			tempuser->name = 0;
		}

		//Set the user's name
		tempuser->name = name;
		//Return the user's ID
		return (tempuser->id);
	}

	//User was not found
	return 0;
}

void cmsusers::setbasepath(char *name){
	basepath = name;
}

int cmsusers::setfileid(int id, char *name){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){

		//Is the user already named?
		if (tempuser->fileid != 0){
			//Remove the old name
			delete tempuser->fileid;
			tempuser->fileid = 0;
		}

		//Set the user's name
		tempuser->fileid = createstring(name);
		//Return the user's ID
		return (tempuser->id);
	}

	//User was not found
	return 0;
}

char *cmsusers::getfileid(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		//Return the user's ID
		return (tempuser->fileid);
	}

	//User was not found
	return 0;
}

int cmsusers::addtext(char *buffer, char *text2add, int pos){
	int cnt = 0;

	while (*(text2add+cnt) != 0x00){
		*(buffer+pos+cnt) = *(text2add+cnt);
		cnt++;
	}
	
	return pos+cnt;
}

void cmsusers::endtext(char *buffer, int pos){
	*(buffer+pos) = 0x00;
}

int cmsusers::setfilename(int id, char *name){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){

		tempuser->filename = name;

		int cnt = 0;
		char *buffer = new char[2000];  //TODO: fix unsafe buffer handling
		cnt = addtext(buffer, basepath, cnt);
		cnt = addtext(buffer, "\\", cnt);
		cnt = addtext(buffer, tempuser->orgname, cnt);
		cnt = addtext(buffer, "\\", cnt);
		cnt = addtext(buffer, name, cnt);
		endtext(buffer, cnt);
		char *tempname=createstring(buffer);
		delete buffer;

		//Is the user already named?
		if (tempuser->savefilename != 0){
			//Remove the old name
			delete tempuser->savefilename;
			tempuser->savefilename = 0;
		}

		//Set the user's name
		tempuser->savefilename = tempname;
		//Return the user's ID
		return (tempuser->id);
	}

	//User was not found
	return 0;
}

int cmsusers::setorgname(int id, char *name){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){

		//Is the user already named?
		if (tempuser->orgname != 0){
			//Remove the old name
			delete tempuser->orgname;
			tempuser->orgname = 0;
		}

		//Set the user's name
		tempuser->orgname = createstring(name);
		//Return the user's ID
		return (tempuser->id);
	}

	//User was not found
	return 0;
}

int cmsusers::setorgid(int id, char *name){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){

		//Is the user already named?
		if (tempuser->orgid != 0){
			//Remove the old name
			delete tempuser->orgid;
			tempuser->orgid = 0;
		}

		//Set the user's name
		tempuser->orgid = createstring(name);
		//Return the user's ID
		return (tempuser->id);
	}

	//User was not found
	return 0;
}

char *cmsusers::getorgname(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		return tempuser->orgname;
	}

	//User was not found
	return 0;
}

char *cmsusers::getfilename(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		return tempuser->savefilename;
	}

	//User was not found
	return 0;
}

char *cmsusers::getdbfilename(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		return tempuser->filename;
	}

	//User was not found
	return 0;
}

char *cmsusers::getorgid(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		return tempuser->orgid;
	}

	//User was not found
	return 0;
}

int cmsusers::getlogonstep(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		return tempuser->logonstep;
	}

	//User was not found
	return -1;
}

void cmsusers::inclogonstep(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		tempuser->logonstep++;
	}
}

void cmsusers::setvalidated(int id, bool validated){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		tempuser->validated = validated;
	}
}

/***************************************************************
 * Method: setpassword
 *    Sets the password for a user given his or her ID
 *    returning the ID for the user modified,
 *    or 0 if the user was not found
 ***************************************************************/

int cmsusers::setpassword(int id, char *password){
	//Find the user to set password
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){

		//Has the user already given a password?
		if (tempuser->password != 0){
			//Remove the old password
			delete tempuser->password;
			tempuser->password = 0;
		}

		//Set the user's password
		tempuser->password = password;
		//Return the user's ID
		return (tempuser->id);
	}

	//User was not found
	return 0;
}

/***************************************************************
 * Method: getname
 *    Returns the name of a user given his or her ID
 *    or 0 if the user was not found
 ***************************************************************/

char *cmsusers::getname(int id){
	//Find the user
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		//Return his or her name
		return tempuser->name;
	}

	//User was not found
	return 0;
}

/***************************************************************
 * Method: getpassword
 *    Returns the password of a user given his or her ID
 *    or 0 if the user was not found
 ***************************************************************/

char *cmsusers::getpassword(int id){
	//Find the user
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		//Return his or her name
		return tempuser->password;
	}

	//User was not found
	return 0;
}

/***************************************************************
 * Method: createszstring
 *    Creates a dynamic string from a constant
 *    TODO: Replace this with proper string handling
 ***************************************************************/

char *cmsusers::createstring(char *instring){
	//Get the input (constant) string's length
	int length = getszlength(instring);

	//Create room for the dynamic string
	char *tempstring = new char[length];

	//Copy the content of the constant string to the dynamic
	for (int i = 0; i<length; i++){
		tempstring[i] = instring[i];
	}

	//Return the pointer to the newly created string
	return tempstring;
}

bool cmsusers::writefile(int id, char *buffer, int *size){
	USER *user = finduser(id);

	if ((user != 0) && user->validated && (user->savefilename != 0)){
		if (user->ofs == 0){
			user->ofs = new ofstream(user->savefilename, ios_base::binary);
		}

		user->ofs->write(buffer, *size);

		return true;
	}

	return false;
}

void cmsusers::setclosed(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		tempuser->ofs->close();
		tempuser->closed = true;
	}
}

bool cmsusers::getclosed(int id){
	//Find the user to name
	LPUSER tempuser = finduser(id);

	//User was found
	if (tempuser != 0){
		return tempuser->closed;
	}
	return true;
}

