#include "shnetfile.h"
#include <windows.h>

#define OFN_DONTADDTORECENT 0x02000000	//Missing from the framework, included here

const char *szWindowClass = "NOCSilentHelperApp";
const char *szWindowName = "NOChan Silent Helper v0.1";
const int RF_SHOW_FILE_DIALOG = 0xA123;

static LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

shnetfile *netfile;		//Class that manages network connections and file transfers

/*
 * getword() picks one and one word from a space-separated list of words
 */

int getword(char *cmdline, int clcnt, char *clbuf, int clbufsize){

	if (cmdline == 0) return -1;

	if (*(cmdline+clcnt) == 0x00) return -1;

	int cnt = 0;
	int ocnt = 0;
	char ch = 0x00;

	while (((cnt-clcnt)<(clbufsize-1)) && ((ch = *(cmdline+clcnt+cnt))!=0x00)){
		if (ch != 0x20){
			*(clbuf+ocnt++) = *(cmdline+clcnt+cnt);
		}
		else{
			*(clbuf+ocnt) = 0x00;
			return clcnt+cnt+1;
		}
		cnt++;
	}

	*(clbuf+ocnt) = 0x00;
	return clcnt+cnt;
}

/*
 * Entry point: WinMain()
 */

int WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow){
	/*
	 * Make sure the application has only one running background instance
	 *  - further instances of this app is communications to the first one
	 */

	//Create a mutex
	HANDLE mutex=CreateMutex(0, true, "mutex_nocshv01_check2xstart");

	if (mutex != 0){

		//Does the mutex already exist?  Then there is already an instance running
		if (GetLastError() == ERROR_ALREADY_EXISTS){

			//Find the window handle to the existant instance
			HWND hwnd = FindWindow(szWindowClass, 0);
			if (hwnd != 0){
				//Send it a message to display a file dialog
				SendMessage(hwnd, RF_SHOW_FILE_DIALOG, 0, 0);
			}
		}

		else {	//This is the first instance of the application

			//Create a class instance to communicate over net and send a file
			netfile = new shnetfile();

			//Fail if the network subsystem will not initialize
			if (!netfile->shdoinitialize()){
				MessageBox (NULL, TEXT("Could not initialize network subsystem\nfor this application."), TEXT("ERROR!"), MB_OK);
				return 0;	
			}

			//Parse the command line
			//TODO: make it more robust
			int clcnt = 0;
			int clwcnt = 0;
			char *clbuf = new char[1024];

			//Get parameter 1
			clcnt = getword(lpCmdLine, clcnt, clbuf, 1024);
			if (clcnt == -1) return 0;  //TODO: display error message
			netfile->shsetserveraddress(clbuf);

			//Get parameter 2
			clcnt = getword(lpCmdLine, clcnt, clbuf, 1024);
			if (clcnt == -1) return 0;  //TODO: display error message
			netfile->shsetserverport(clbuf);

			//Get parameter 3
			clcnt = getword(lpCmdLine, clcnt, clbuf, 1024);
			if (clcnt == -1) return 0;  //TODO: display error message
			netfile->shsetusername(clbuf);

			//Get parameter 4
			clcnt = getword(lpCmdLine, clcnt, clbuf, 1024);
			if (clcnt == -1) return 0;  //TODO: display error message
			netfile->shsetpassword(clbuf);

			//Confirm that there are no fifth parameter
			clcnt = getword(lpCmdLine, clcnt, clbuf, 1024);
			if (clcnt != -1) return 0;  //TODO: display error message

			delete clbuf;

			//Register an instance of the window class
			WNDCLASSEX wndClassex;

			if (!hPrevInstance){
				wndClassex.cbSize = sizeof(WNDCLASSEX);
				wndClassex.style = CS_DBLCLKS | CS_OWNDC;
				wndClassex.lpfnWndProc = (WNDPROC) WindowProc;
				wndClassex.cbClsExtra = 0;
				wndClassex.cbWndExtra = 0;
				wndClassex.hInstance = hInstance;
				wndClassex.hIcon = 0;
				wndClassex.hCursor = 0;
				wndClassex.hbrBackground = (HBRUSH) COLOR_BACKGROUND;
				wndClassex.lpszMenuName = 0;
				wndClassex.lpszClassName = szWindowClass;
				wndClassex.hIconSm = 0;

				RegisterClassEx(&wndClassex);	//TODO: test if this fails and end the program (should rarely happen)
			}

			//Create a window (although invisible) for this application)
			HWND hWnd = CreateWindowEx(WS_EX_OVERLAPPEDWINDOW | WS_EX_CLIENTEDGE, szWindowClass, szWindowName,
				WS_OVERLAPPEDWINDOW | WS_BORDER | WS_CAPTION, 100, 100, 500, 400, 0, 0, hInstance,0);

			//Enter the message loop
			MSG msg;	//to get messages
			bool action = false;	//To track activity (if no activity, Sleep() to relieve system)
			bool quit = false;	//loop until quit
			while(!quit){
				//Check if there are messages in the queue and get it if so
				if (PeekMessage(&msg, hWnd, 0, 0, PM_REMOVE) != 0){
					action = true;	//Set that something has happened this run

					//Check for important messages (that is WM_QUIT)
					switch (msg.message){
						case WM_QUIT:
							quit = true;
							break;
						default:
							break;
					}
					//Send the message further
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}

				//Send data from file if there are any
				if (netfile->shdosenddata()){
					action = true;  //Set that something has happened this run
				}

				//Did something happen?
				if (!action) {
					Sleep(100);	//no, let's Sleep()
				}
				action = false;	//Prepare for next run
			}

			//Release mutex to allow for a new main instance after this one closes
			CloseHandle(mutex);

			//Exit program
			return msg.wParam;

		}

	}

	//Exit program
	return 0;
}

static LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam){
	char *fname;	//make global?
	OPENFILENAME ofn; //make global?
	PAINTSTRUCT ps;
	HDC hdc;

	switch (uMsg){
		case WM_CREATE:
			//Initialize the form (we don't need any, therefore empty)
			return 0;
 
		case RF_SHOW_FILE_DIALOG:	//Select and upload file
			if (!netfile->issendingfile()){	//If not already sending a file

				//Prepare the file dialog
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize = sizeof(OPENFILENAME);
				fname = new char[1024];
				fname[0] = 0x00;
				ofn.lpstrFile = fname;
				ofn.nMaxFile = 1024;
				ofn.lpstrInitialDir = "c:\\";
				ofn.lpstrTitle = "Velg fil for opplasting";
				ofn.Flags = OFN_DONTADDTORECENT | OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_NONETWORKBUTTON | OFN_PATHMUSTEXIST;

				//Get a file name from the file dialog
				if (GetOpenFileName(&ofn)){
					netfile->shsetfilename(fname);	//Tell the net and file system the file name
					netfile->shsetfilepathname((fname+ofn.nFileOffset));	//Tell the net and file system the file name
					netfile->shdosendfile();		//Tell the net and file system to send it
				}

				delete fname;
			}

			return 0;

		case WM_PAINT:  // validate the window
			hdc = BeginPaint(hwnd, &ps);	//Dummy
			EndPaint(hwnd, &ps);			//Dummy
			return 0; 
 
		case WM_CLOSE:
			//Fallthrough
		case WM_DESTROY:
			PostQuitMessage(0);	//Tell the application to quit
			return 0;

		default:;
    } 
	return DefWindowProc(hwnd, uMsg, wParam, lParam);
}
