#include "shnetfile.h"

//Constructor
shnetfile::shnetfile(){
	serveraddress = 0;
	filename = 0;
	username = 0;
	password = 0;
	filesize = 0;
	connected = false;
	initialized = false;
	sendingfile = false;
	fileopened = false;
	loggedon = false;
	error = 0;
	ifs = 0;
	buffer = 0;
}

//Destructor
shnetfile::~shnetfile(){
	shdodeinitialize();
}

//Initialize network subsystem
bool shnetfile::shdoinitialize(){
	if (!initialized){
		error = WSAStartup (0x0202, &w);

		if (error){
			return false;
		}

		initialized = true;

		if (w.wVersion != 0x0202){
			shdodeinitialize();
			return false;
		}
	}

	return true;
}

//Deinitialize the network subsystem
bool shnetfile::shdodeinitialize(){
	if (connected) shdodisconnect();
	if (initialized){
		WSACleanup ();
		initialized = false;
	}
	return true;	//This will "never" fail (it doesn't matter anyway)
}

//Connect to the server
bool shnetfile::shdoconnect(){
	if (!connected){

		if (!initialized){
			if (!shdoinitialize()) return false;
		}

		s = socket (AF_INET, SOCK_STREAM, 0);

		target.sin_family = AF_INET;
		target.sin_port = htons (atoi(shgetserverport())); //Port number to connect to server
//		target.sin_addr.s_addr = inet_addr ("129.241.208.139");
		target.sin_addr.s_addr = inet_addr (shgetserveraddress());
		if (connect(s, (SOCKADDR *) &target, sizeof(target)) == SOCKET_ERROR){
			shdodeinitialize();
			return false;
		}

		connected = true;
	}

	return true;
}

//Disconnect from the server
bool shnetfile::shdodisconnect(){
	//Check for logon and file open
	if (connected){
			shutdown (s, SD_SEND);
			closesocket (s);
			connected = false;
	}

	return true;
}

void shnetfile::shsetfilename(char *fn){
	filename = createstring(fn);
}

void shnetfile::shsetfilepathname(char *fpn){
	filepathname = createstring(fpn);
}

void shnetfile::shsetserveraddress(char *sa){
	serveraddress = createstring(sa);
}

void shnetfile::shsetserverport(char *p){
	serverport = createstring(p);
}

char *shnetfile::shgetserveraddress(){
	return serveraddress;
}

char *shnetfile::shgetserverport(){
	return serverport;
}

char *shnetfile::shgetfilename(){
	return filename;
}

void shnetfile::shsetusername(char *un){
	username = createstring(un);
}

void shnetfile::shsetpassword(char *pwd){
	password = createstring(pwd);
}

//Signal that a file is selected and is ready for transfer
void shnetfile::shdosendfile(){
	if ((filename != 0) && (filepathname != 0) && (username != 0) && (password !=0) && (serveraddress != 0) && (serverport != 0)){
		sendingfile = true;
	}
}

//Check if a file is flagged as "sendable"
bool shnetfile::issendingfile(){
	return sendingfile;
}

//Log on to the connected server
bool shnetfile::shdologon(){
	if (!connected){
		if (!shdoconnect()) return false;
	}

	//TODO: proper error checking
	if (!loggedon){
		send (s, username, getszlength(username), 0);
		Sleep(250);
		send (s, password, getszlength(password), 0);
		Sleep(250);
		send (s, filepathname, getszlength(filepathname), 0);
		Sleep(250);
		send (s, "0\n", 2, 0);
		Sleep(250);
		loggedon = true;
	}

	return true;
}

//Open the file to send
bool shnetfile::shdoopenfile(){
	if (!fileopened){
		ifs = new ifstream(filename, ios_base::binary);
		buffer = new char[1024];

		fileopened = true;
	}

	return true;
}

//Send a portion of a file
bool shnetfile::shdosenddata(){
	if (!sendingfile){
		return false;
	}

	if (!loggedon){
		if (!shdologon()) return false;
	}

	if (!fileopened){
		if (!shdoopenfile()) return false;
	}

	//No else here! This is on purpose!  Think about it!

	if (fileopened){

		if (!ifs->good()){
			//TODO: tidy up this section
			loggedon = false; //dologoff()
			sendingfile = false; //dontsendfile()
			fileopened = false;
			delete buffer;
			buffer = 0;
			delete filename;
			filename = 0;
			delete filepathname;
			filepathname = 0;
			shdodisconnect();
			ifs->close();
			ifs = 0;
			return false;
		}

		ifs->read(buffer, 1024);
		if (send (s, buffer, 1024, 0) == SOCKET_ERROR) MessageBox (NULL, "SOCKET_ERROR", TEXT("Socket error"), MB_OK);
//		Sleep(100);

	}

	return true;
}

/***************************************************************
 * Method: getszlength
 *    Returns the length of the given zero terminated string
 *    including the zero itself, 0x0a is converted to 0x00
 ***************************************************************/

static int getszlength(char *instring){
	//In case of null pointer input
	if (instring == 0) return 0;

	//Search from the beginnig of the string
	int count = 0;

	//Until first occurence of a null character
	char ch = 0x00;
	do{
		ch = *(instring+(count++));
	} while ((ch != '\0') && (ch != 0x0a));

	*((instring+(count))-1) = 0x00;

	//return the found count (plus one)
	return count+1;
}

/***************************************************************
 * Method: createszstring
 *    Creates a dynamic string from a constant
 *    TODO: Replace this with proper string handling
 ***************************************************************/

static char *createstring(char *instring){
	//Get the input (constant) string's length
	int length = getszlength(instring);

	//Create room for the dynamic string
	char *tempstring = new char[length];

	//Copy the content of the constant string to the dynamic
	for (int i = 0; i<length; i++){
		tempstring[i] = instring[i];
	}

	//Return the pointer to the newly created string
	return tempstring;
}
