{****************************************************
 *
 * sgen - script generator
 *
 * sgen (c) 2004 av Erik Vold,
 * laget av Erik Vold for pen Kanal Norge i 2006
 *
 ***************************************************}

//{$DEFINE Debug}

program sgen;

uses
  crt, mysql, sysutils;

{****************************************************
 *
 * K O N F I G U R A S J O N S D A T A
 *
 ***************************************************}

type
  configt=record
    verbose:string;
    local_path_relative:string;
    local_root_to_work_dir:string;
    path_for_storing_scripts:string;
    ip_for_db:string;
    name_for_db:string;
    login_name_for_db:string;
    pwd_for_db:string;
    ip_for_ftp_in:string;
    login_name_for_ftp_in:string;
    pwd_for_ftp_in:string;
    local_path_for_ftp_in:string;
    ip_for_ftp_out:string;
    login_name_for_ftp_out:string;
    pwd_for_ftp_out:string;
    local_path_for_ftp_out:string;

    def_path_to_store_files_on_remote_ftp:string;
    def_path_to_get_files_on_remote_ftp:string;

    number_of_bitrates:integer;
    bitrates:array[0..9] of string;

    local_path_xml_templates:string;
    number_of_xml_templates:integer;
    xml_template:array[0..9] of string;
    
    org_name:string;
    file_name:string;
    content_id:string;
  end;

{****************************************************
 *
 * G L O B A L E   V A R I A B L E R
 *
 ***************************************************}

var
  line_count:integer;  //For  indikere hvilken linje i konfig fil noe feiler


{****************************************************
 *
 * initconfig()
 *   inndata: en "record" med konfigureringsdata
 *
 * Brukes for  initialisere konfigureringsstrukturen.
 *
 ***************************************************}

procedure initconfig(var cfg:configt);
var
  lp:integer;
begin
  with (cfg) do
  begin
    verbose:='';
    local_path_relative:='';
    local_root_to_work_dir:='';
    path_for_storing_scripts:='';
    ip_for_db:='';
    pwd_for_db:='';
    ip_for_ftp_in:='';
    login_name_for_ftp_in:='';
    pwd_for_ftp_in:='';
    local_path_for_ftp_in:='';
    ip_for_ftp_out:='';
    login_name_for_ftp_out:='';
    pwd_for_ftp_out:='';
    local_path_for_ftp_out:='';
    name_for_db:='';
    login_name_for_db:='';

    def_path_to_store_files_on_remote_ftp:='';
    def_path_to_get_files_on_remote_ftp:='';

    number_of_bitrates:=0;
    for lp:=0 to 9 do
    begin
      bitrates[lp]:='';
    end;

    local_path_xml_templates:='';
    number_of_xml_templates:=0;
    for lp:=0 to 9 do
    begin
      xml_template[lp]:='';
    end;
    
    org_name:='';
    file_name:='';
    content_id:='';
  end;
end;

{****************************************************
 *
 * stripext()
 *   inndata: et filnavn
 *   utdata : Samme filnavn uten extention
 *
 * Bruker for  fjerne filextention
 *
 ***************************************************}

function stripext(file_name:string):string;
var
  tmpstr:string;
  lp:integer;
  dotfound:boolean;
begin
  tmpstr:='';
  dotfound:=false;
  
  if (length(file_name)>1) then
  begin

    for lp:=length(file_name) downto 1 do
    begin
      if (dotfound) then tmpstr:=file_name[lp]+tmpstr;
      
      if (file_name[lp]='.') then dotfound:=true;
    end;

  end;
  
  stripext:=tmpstr;
end;

{****************************************************
 *
 * get_number_of_tokens()
 *   inndata: en linje med tekst
 *   utdata : antall "ord" i en setning.
 *
 * Brukes for  returnere antall tokens i en streng.
 *
 ***************************************************}

function get_number_of_tokens(line:string):integer;
//Returnerer antall "ord" i en setning (tekststreng)
var
  last_space:boolean;  //Siste tegnet lest er mellomrom
  lp:integer;  //Teller for lkke
  cnt:integer; //Antall tokens
  quote:boolean;  //Vi er inne i en "quote", mellomrom skal behandles annerledes
begin
  cnt := 0;
  last_space := true;
  quote:=false;

  //Hopp over linjer med kommentarer eller tomme linjer
  if (line[1] <> '#') and (line <> '') then
  begin
 
    //Tell ord
    for lp:=1 to length(line) do
    begin

      //Bytt oppfrsel for hvert anfrselstegn som er funnet,
      //mellomrom som er mellom et "quote"-par skal ikke regnes som skille mellom "tokens"
      if (line[lp] = '"') then
      begin
        if (quote) then quote:=false else quote:=true;
      end;

      //Dersom vi har noe annet enn et mellomrom og at forrige tegn var mellomrom
      if (line[lp] <> ' ') and (last_space) then
      begin
        //og vi ikke befinner oss i en "quote",
        if (not quote) then
        begin
          //s har vi et nytt token
          inc (cnt);
          last_space := false;  //Dette tegnet var ikke mellomrom
        end;
      end else
      begin
        if (line[lp] = ' ') then last_space := true;  //Dette tegnet var mellomrom
      end;
    end;
  end;
  
  get_number_of_tokens := cnt;
end;

{****************************************************
 *
 * get_token()
 *   inndata: en linje med tekst,
 *            en indeks for hvilket token i tekst som skal returneres
 *   utdata : "ordet" som er hentet ved indeks n
 *
 * Brukes for  hente et spesifikt "ord" fra en streng
 *
 ***************************************************}

function get_token(line:string; index:integer):string;
//Henter et "ord" fra en setning (tekststreng)
var
  last_space:boolean;  //Siste tegnet lest er mellomrom
  lp:integer;  //Teller for lkke
  cnt:integer; //Antall tokens
  tmp_token:string;  //Mellomlager for hentet "token"
  quote:boolean;  //Vi er inne i en "quote", mellomrom skal behandles annerledes
begin
  quote:=false;
  tmp_token:='';
  
  cnt := 0;
  last_space := true;

  //Hopp over linjer med kommentarer eller tomme linjer
  if (line[1] <> '#') and (line <> '') then
  begin
 
    for lp:=1 to length(line) do
    begin

      //Bytt oppfrsel for hvert anfrselstegn som er funnet
      if (line[lp] ='"') then
      begin
        if quote then quote:=false else quote:=true;
      end;

      //Dersom vi har noe annet enn et mellomrom, eller vi befinner oss inne i en "quote"
      if (line[lp] <> ' ') or (quote) then
      begin
        //og forrige tegnet var mellomrom
        if (last_space) then
        begin
          //S har vi et nytt "ord" eller "token"
          inc (cnt);
          last_space := false;  //Dette tegnet var ikke mellomrom
        end;

	//N har vi funnet "ordet" som vi nsker  hente, ta vare p det!
        if (cnt = index) and (line[lp]<>'"') then tmp_token := tmp_token + line[lp];
      end else
      begin
        if (line[lp] = ' ') then last_space := true;  //Dette tegnet var mellomrom
      end;
    end;
  end;
  
  get_token := tmp_token;
end;

{****************************************************
 *
 * translate()
 *   inndata: en linje med tekst,
 *            konfigureringsstruktur som skal fylles med data fra linje med tekst
 *
 * Brukes for  oversette linjer fra konfigureringsfil for  fylle en konfigureringsstruktur
 *
 ***************************************************}

procedure translate(line:string; var cfg:configt);
//Parser konfigureringsfila
var
  number_of_tokens:integer;
  token1, token2, token3:string;
  num:integer;
  err:word;
begin
  number_of_tokens := get_number_of_tokens(line);

  //Initialiser tokens
  token1:='';
  token2:='';
  token3:='';

  //Sikre at det er minst to tokens fr vi gidder  gjre noe
  if (number_of_tokens>1) then
  begin

    with (cfg) do
    begin
    
      if (number_of_tokens>=1) then token1 := get_token(line, 1);
      if (number_of_tokens>=2) then token2 := get_token(line, 2);
      if (number_of_tokens>=3) then token3 := get_token(line, 3);

{$IFDEF DEBUG}
//*********  For debugging:  *********
    writeln (number_of_tokens, ' tokens: ', token1, ', ', token2, ', ', token3);
{$ENDIF}

      //Parametre som utgjr 2 tokens
      if (token1='verbose') then verbose := token2 else
      if (token1='local_path_relative') then local_path_relative := token2 else
      if (token1='local_root_to_work_dir') then local_root_to_work_dir := token2 else
      if (token1='path_for_storing_scripts') then path_for_storing_scripts := token2 else
      if (token1='ip_for_db') then ip_for_db := token2 else
      if (token1='pwd_for_db') then pwd_for_db := token2 else
      if (token1='ip_for_ftp_in') then ip_for_ftp_in := token2 else
      if (token1='login_name_for_ftp_in') then login_name_for_ftp_in := token2 else
      if (token1='pwd_for_ftp_in') then pwd_for_ftp_in := token2 else
      if (token1='local_path_for_ftp_in') then local_path_for_ftp_in := token2 else
      if (token1='ip_for_ftp_out') then ip_for_ftp_out := token2 else
      if (token1='login_name_for_ftp_out') then login_name_for_ftp_out := token2 else
      if (token1='pwd_for_ftp_out') then pwd_for_ftp_out := token2 else
      if (token1='local_path_for_ftp_out') then local_path_for_ftp_out := token2 else
      if (token1='name_for_db') then name_for_db := token2 else
      if (token1='login_name_for_db') then login_name_for_db := token2 else
      if (token1='local_path_xml_templates') then local_path_xml_templates:= token2 else
      if (token1='def_path_to_store_files_on_remote_ftp') then def_path_to_store_files_on_remote_ftp:=token2 else
      if (token1='def_path_to_get_files_on_remote_ftp') then def_path_to_get_files_on_remote_ftp:=token2 else

      if (number_of_tokens>2) then
      begin
	
	if (token1='bitrate') then
	begin
          //Oversett tekst til nummer
	  val(token2, num, err);
	  
	  //Dersom oversettelsen feilet
	  if (err<>0) then
	  begin
            //Informer om at token2 ikke kunne oversettes til et tall
            writeln('Linje ', line_count, ', translate(), feil, token2 er ikke et nummer');
	  end else
	  begin
	    //Oversettelsen gikk bra
	    
	    //Ta vare p info om bitrate
	    bitrates[number_of_bitrates]:=token3;
	    
	    //reflekter antallet ved  ke telleren
	    inc (number_of_bitrates);
	  end;
          
	end else

	if (token1='xml_template') then
	begin
          //Oversett tekst til nummer
	  val(token2, num, err);
	  
	  //Dersom oversettelsen feilet
	  if (err<>0) then
	  begin
            //Informer om at token2 ikke kunne oversettes til et tall
            writeln('Linje ', line_count, ', translate(), feil, token2 er ikke et nummer');
	  end else
	  begin
	    //Oversettelsen gikk bra
	    
	    //Ta vare p info om filnavn til XML maler
	    xml_template[number_of_xml_templates]:=token3;
	    
	    //reflekter antallet ved  ke telleren
	    inc (number_of_xml_templates);
	  end;
        end else

	begin
          //Informer om at det er parametre i konfigurasjonsfila som ikke ble forsttt
          writeln('Linje ', line_count, ', translate(), feil, forstr ikke ', token1);
	end;
	
      end else
      begin
        //Informer om at det er parametre i konfigurasjonsfila som ikke ble forsttt
        writeln('Linje ', line_count, ', translate(), feil, forstr ikke ', token1);
      end;
    end;
   
  end;
end;

{****************************************************
 *
 * randname()
 *   utdata : en 16 tegn lang tilfeldig alfanumerisk streng
 *
 * Brukes for  generere del av et tilfeldig filnavn
 *
 ***************************************************}

function randname():string;
var
  lp:integer;
  tmpstr:string;
  bt:byte;
begin
  tmpstr:='';

  for lp:=1 to 16 do
  begin
    bt:=trunc(random()*26);
    tmpstr:=tmpstr+chr(bt+ord('a'));
  end;
  randname:=tmpstr;
end;

{****************************************************
 *
 * read_cfg_file()
 *   inndata: et filnavn til en konfigureringsfil,
 *            struktur for  ta i mot konfigureringsdata
 *   utdata : feilkode.  0=ingen feil
 *
 * Brukes for  lese inn en konfigureringsfil.
 *
 ***************************************************}

function read_cfg_file(cfg_file_name:string; var cfg:configt):integer;
//Leser konfigureringsfila linje for linje
var
  line:string;  //Holder en og en lest linje
  cfg_file:text;  //Filehndterer for konfigureringsfil
  result:integer;  //Resultat av i/u-kommandoer
begin
  read_cfg_file := 0;  //Initielt s har lesing gtt greit
  initconfig(cfg);  //initialiser konfigurasjonsstruktur

  assign(cfg_file, cfg_file_name);
  {$i-}  //Sl av i/u-feilsjekking
  reset(cfg_file);  //pne fil for lesing
  {$i+}  //Sl p i/u-feilsjekking

  result := ioresult;
  if (result=0) then  //pning av fil ok
  begin
    while (not eof(cfg_file)) do  //S lenge det er konfigurasjonsparametre  lese
    begin
      //Hold orden p hvilken linje i fil som blir prossessert i yeblikket
      inc (line_count);

      {$i-}
      readln(cfg_file, line);  //Les en linje
      {$i+}
      
      result := ioresult;
      if (ioresult=0) then
      begin
        translate(line, cfg)  //Oversett og fyll inn i konfig-objektet
      end else
      begin
      	//Lesing fra fil feilet av en eller annen mystisk grunn.
      	//Dette vil sannsynligvis vre en meget sjelden hendelse, men 
      	//det br hndteres.
        read_cfg_file := result;  //Feilkode som skal returneres
        break;  //Bryt ut av "while"-lkke.
      end;
    end;
    
    close(cfg_file);  //Steng konfigurasjonsfil
  end;

  read_cfg_File := result;  //Returner resultat av siste I/U-operasjon
end;

{****************************************************
 *
 * verify_config()
 *   inndata: konfigurasjonsdata
 *
 * Kontroll av de viktigste parametrene
 *
 ***************************************************}

procedure verify_config(cfg:configt);
begin
  with (cfg) do
  begin
  
    if (number_of_bitrates=0) then
    begin
      //Informer om at det ikke er definert noen bitrater
      writeln('FEIL:  Ingen bitrater er definert!');
      halt(0);  //Ingen vits i  fortsette nr man ikke kan gjre noe
    end else
  
    if (number_of_bitrates<>number_of_xml_templates) then
    begin
      //Informer om at det ikke er definert tilsvarende templates for bitrater
      writeln('FEIL:  #bitrates != #templates');
      halt(0);  //Ingen vits i  fortsette nr man ikke kan gjre noe
    end;

  end;
end;

{****************************************************
 *
 * gen_file_ftp_in()
 *   inndata: 
 *
 * Genererer script for mottak av filer over ftp
 *
 ***************************************************}

procedure gen_file_ftp_in(rnd_str:string; cfg:configt);
var
  textfile:text;
  result:integer;
  local_path:string;
  local_file_name:string;
begin
  with (cfg) do
  begin

    //Genererer filnavn
    local_file_name:=org_name + '_' + rnd_str + '_in.txt';

    if (local_path_relative='true') then
    begin
      //Lokale stier er relative
      local_path:=local_root_to_work_dir + '\';
    end else
    if (local_path_relative='false') then
    begin
      //Lokale stier er absolutte
      local_path:='';
    end else
    begin
      //Feil verdi i variabel local_path_relative
      writeln('ERROR: local_path_relative inneholder en ulovlig verdi');
      halt(0);
    end;
 
    assign (textfile, local_path+path_for_storing_scripts+'\'+local_file_name);
    {$i-}
    rewrite(textfile);
    {$i+}
    result:=ioresult;

    //TODO:  Feilsjekking
  
    {$i-}
    writeln(textfile, 'open ' + ip_for_ftp_in);
    writeln(textfile, login_name_for_ftp_in);
    writeln(textfile, pwd_for_ftp_in);
    writeln(textfile, 'cd "' + def_path_to_get_files_on_remote_ftp + '"');
    writeln(textfile, 'cd "' + org_name + '"');
    writeln(textfile, 'lcd "' + local_path + local_path_for_ftp_in + '"');

{$IFDEF DEBUG}
//*********  For debugging:  *********
    writeln(textfile, 'hash');
{$ENDIF}

    writeln(textfile, 'get "' + file_name + '"');
    writeln(textfile, 'quit');
    {i+}

    //TODO:  Feilsjekking

    //Vi er ferdige med fila n
    close(textfile);
  end;
end;

{****************************************************
 *
 * gen_file_ftp_out()
 *   inndata: 
 *
 * Genererer script for sending av filer over ftp
 *
 ***************************************************}

procedure gen_file_ftp_out(lp:integer; rnd_str:string; cfg:configt);
var
  textfile:text;
  result:integer;
  local_path:string;
  local_file_name:string;
begin
  with (cfg) do
  begin

    //Genererer filnavn
    local_file_name:=org_name + '_' + rnd_str +'_'+ bitrates[lp] +'_out.txt';

    if (local_path_relative='true') then
    begin
      //Lokale stier er relative
      local_path:=local_root_to_work_dir + '\';
    end else
    if (local_path_relative='false') then
    begin
      //Lokale stier er absolutte
      local_path:='';
    end else
    begin
      //Feil verdi i variabel local_path_relative
      writeln('ERROR: local_path_relative inneholder en ulovlig verdi');
      halt(0);
    end;

    assign (textfile, local_path+path_for_storing_scripts+'\'+local_file_name);
    {$i-}
    rewrite(textfile);
    {$i+}
    result:=ioresult;
  
    {$i-}
    writeln(textfile, 'open '+ip_for_ftp_out);
    writeln(textfile, login_name_for_ftp_out);
    writeln(textfile, pwd_for_ftp_out);

{$IFDEF DEBUG}
//*********  For debugging:  *********
    writeln(textfile, 'hash');
{$ENDIF}

    writeln(textfile, 'cd "'+def_path_to_store_files_on_remote_ftp+'"');
    writeln(textfile, 'cd "'+org_name+'"');
    writeln(textfile, 'cd "'+bitrates[lp]+'"');
    writeln(textfile, 'lcd "'+local_path+local_path_for_ftp_out+'"');
    writeln(textfile, 'put "'+stripext(file_name)+'.mp4"');
    writeln(textfile, 'cd ..');
    writeln(textfile, 'bye');
    {i+}
    result:=ioresult;
    close(textfile);
  end;
end;

{****************************************************
 *
 * gen_file_bat()
 *   inndata: 
 *
 * Genererer script for utfrsel av jobber
 *
 ***************************************************}

procedure gen_file_bat(rnd_str:string; cfg:configt);
var
  textfile:text;
  result:integer;
  local_path:string;
  lp:integer;
  input_file, output_file, preset_file:string;
begin
  with (cfg) do
  begin

    if (local_path_relative='true') then
    begin
      //Lokale stier er relative
      local_path:=local_root_to_work_dir + '\';
    end else
    if (local_path_relative='false') then
    begin
      //Lokale stier er absolutte
      local_path:='';
    end else
    begin
      //Feil verdi i variabel local_path_relative
      writeln('ERROR: local_path_relative inneholder en ulovlig verdi');
      halt(0);
    end;

    assign (textfile, local_path+'\'+path_for_storing_scripts+'\'+org_name+'_'+rnd_str+'.bat');
    {$i-}
    rewrite(textfile);
    {$i+}
    result:=ioresult;

    {$i-}
    writeln(textfile, 'd:');
    writeln(textfile, 'cd ' + local_root_to_work_dir);
    writeln(textfile, local_root_to_work_dir+'\gsstart ' + content_id + ' ' + rnd_str);
    writeln(textfile, 'cd ' + local_root_to_work_dir+ '\' + path_for_storing_scripts);
    writeln(textfile, 'FTP -s:"'+org_name+'_'+rnd_str+'_in.txt"');
    writeln(textfile, 'del "'+org_name+'_'+rnd_str+'_in.txt"');

    //Generer utfrsel av encoding for alle bitrater
    for lp:=0 to (number_of_bitrates-1) do
    begin
      //Ordne filnavn
      input_file:=local_path+local_path_for_ftp_in+'\'+file_name;
      output_file:=local_path+local_path_for_ftp_out+'\'+stripext(file_name)+'.mp4';
      preset_file:=local_path+local_path_xml_templates+'\'+xml_template[lp];

      writeln(textfile, 'c:');
      writeln(textfile, 'cd "\Program Files\Envivio\Envivio Encoding Station 2.5"');
      writeln(textfile, 'ees_cmd -i "' + input_file + '" -o "' + output_file + '" -p "' + preset_file + '"');
      writeln(textfile, 'd:');
      writeln(textfile, 'cd ' + local_root_to_work_dir+ '\' + path_for_storing_scripts);
      writeln(textfile, 'FTP -s:"'+org_name+'_'+rnd_str+'_'+bitrates[lp]+'_out.txt"');
//      writeln(textfile, 'del "'+local_path+path_for_storing_scripts+'\'+org_name+'_'+rnd_str+ '_' + bitrates[lp] +'.xml"');
      writeln(textfile, 'del "'+org_name+'_'+rnd_str+'_'+bitrates[lp]+'_out.txt"');
    end;

    writeln(textfile, local_root_to_work_dir+'\gsstop '+content_id);
    {i+}
    result:=ioresult;
    close(textfile);
  end;
end;

{****************************************************
 *
 * process_new_files()
 *   inndata: struktur for  ta i mot konfigureringsdata
 *
 * Generer script ut i fra nye filer som er lastet opp
 *
 * TODO:  Analysere og se om denne kan "plukkes" fra hverandre og kortes ned
 *
 ***************************************************}

procedure process_new_files(cfg:configt);
var
  sock:pmysql;
  qmysql:tmysql;
  recbuf, recbuf2:pmysql_res;
  rowbuf, rowbuf2:tmysql_row;

  ip_address:pchar;
  login_name:pchar;
  pass_word:pchar;
  db_name:pchar;

  query:string;
  pquery:pchar;
  
  lp:integer;
  rnd_str:string;
begin

{$IFDEF DEBUG}
//*********  For debugging:  *********
  Write ('Kobler til mySQL: ');
{$ENDIF}

  //Konverter fra string til pchar
  ip_address:=nil;
  login_name:=nil;
  pass_word:=nil;
  db_name:=nil;

  //Alloker buffer til pchar
  ip_address:=stralloc(250);
  login_name:=stralloc(250);
  pass_word:=stralloc(250);
  db_name:=stralloc(250);

  //Sikker konvertering, max 250 tegn
  strplcopy(ip_address, cfg.ip_for_db, 250);
  strplcopy(login_name, cfg.login_name_for_db, 250);
  strplcopy(pass_word, cfg.pwd_for_db, 250);
  strplcopy(db_name, cfg.name_for_db, 250);

  //Koble til databaseserver
  sock :=  mysql_connect(pmysql(@qmysql), ip_address, login_name, pass_word);

  if (sock=nil) then
  begin
    writeln(stderr, 'Tilkobling til mySQL feilet.');
    writeln(stderr, mysql_error(@qmysql));
    halt(0);
  end;
  
{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('OK');
  writeln();
{$ENDIF}

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln ('Host info       : ', mysql_get_host_info(sock));
  writeln ('Server info     : ', mysql_stat(sock));
  writeln ('Client info     : ', mysql_get_client_info);
{$ENDIF}

{$IFDEF DEBUG}
//*********  For debugging:  *********
   write ('Velger database: ');
{$ENDIF}

  //Velger database
  if (mysql_select_db(sock, db_name)<0) then
  begin
    writeln(stderr, 'Klarte ikke  velge database ', db_name);
    writeln(stderr, mysql_error(sock));
    halt (0);
  end;

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('OK');
  writeln();
{$ENDIF}

  //Utfrer sprring etter nye filer
  query:='select orgid, filename, contentid from content where fin_upload="yes" and gen_script="no"';

  //konverterer fra string til pchar
  pquery:=nil;
  pquery:=stralloc(250);
  strplcopy(pquery, query, 250);

{$IFDEF DEBUG}
//*********  For debugging:  *********
  write('Utfrer sprring etter nye filer: ');
{$ENDIF}

  if (mysql_query(sock,pquery) < 0) then
  begin
    writeln(stderr, 'Sprring etter filliste feilet');
    writeln(stderr, mysql_error(sock));
    halt(0);
  end;

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('OK');
  writeln();
{$ENDIF}

{$IFDEF DEBUG}
//*********  For debugging:  *********
  write('"Parser" sprring: ');
{$ENDIF}

  //Lagre resultatet av sprring i buffer
  recbuf:=mysql_store_result(sock);
  if recbuf=nil then
  begin
    writeln ('Ingen nye filer funnet!');
    mysql_close(sock);
    halt(0);
  end;

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('OK');
  writeln();
{$ENDIF}

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('Antall rader returnert: ', mysql_num_rows(recbuf));
  writeln('Antall felter per rad : ', mysql_num_fields(recbuf));
  writeln();
{$ENDIF}

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln ('Liste over nye filer:');
{$ENDIF}

  //Hent en rad
  rowbuf := mysql_fetch_row(recbuf);

  //S lenge det er rader, prossesser filer
  while (rowbuf<>nil) do
  begin

    //Sprring for  f tak i organisasjonsnavn (eier av denne fila)
    query:='select orgnavn from organisasjoner where orgid="'+rowbuf[0]+'"';
    
    //Konverter fra string til pchar - allokering er allerede gjort
    strplcopy(pquery, query, 250);

{$IFDEF DEBUG}
//*********  For debugging:  *********
  write('Utfrer sprring etter nye organisasjonsnavn: ');
{$ENDIF}

    if (mysql_query(sock, pquery)<0) then
    begin
      //Sprring feilet
      writeln('Sprring etter organisasjon med id ', rowbuf[0], ' feilet');
      writeln();

      //Resultatet dersom dette skjer er at scripts ikke blir generert for denne fila,
      //  nytt forsk neste gang.
      //TODO:  Kanskje logge dette p en eller annen mte?  Er DB tilgjengelig i og med at sprring feilet?
    end else
    begin
      
{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('OK');
{$ENDIF}

      //"parse" resultatet av sprringen
      recbuf2:=mysql_store_result(sock);

      //Dersom sprringen ledet til noe resultat
      if (recbuf2<>nil) then
      begin
        rowbuf2:=mysql_fetch_row(recbuf2);

	//Dersom rowbuf ikke returnerer blankt
        if (rowbuf2<>nil) then
        begin
          //Ta vare p organisasjonsnavnet
          cfg.org_name:=rowbuf2[0];

          //Ta vare p content id
          cfg.content_id:=rowbuf[2];

          //Frigjr ressurser som ikke lenger behves
          mysql_free_result(recbuf2);

//**
//* Her har vi all informasjonen som vi trenger for  generere 
//* de filene vi har behov for.                               
//**
//********************************************************
//********************************************************
//********************************************************
//********************************************************
//********************************************************
//********************************************************

          //Genererer tilfeldig del av et filnavn
          rnd_str:=randname();

          //Tar vare p filnavn fra databasen
          cfg.file_name:=rowbuf[1];

          //Genererer jobbfil
          gen_file_bat(rnd_str, cfg);

          //Generer fil for nedlasting over FTP
          gen_file_ftp_in(rnd_str, cfg);
          
{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('Tilfeldig del av filnavn: ', rnd_str);
{$ENDIF}

          //Vi vet helt sikkert at det eksisterer bitrater i listen
          //For hver bitrate som er definert
          for lp:=0 to (cfg.number_of_bitrates-1) do
          begin
            //Generer fil for opplastning over FTP
            gen_file_ftp_out(lp, rnd_str, cfg);
          end;


        //Sett flagg at script for denne fila er generert
        //Sprring for  f tak i organisasjonsnavn (eier av denne fila)
        query:='update content set gen_script="yes" where contentid="' + cfg.content_id + '"';
    
        //Konverter fra string til pchar - allokering er allerede gjort
        strplcopy(pquery, query, 250);
        
        if (mysql_query(sock, pquery)<0) then
        begin
          //Sprring feilet
          writeln('kunne ikke sette status "gen_script=yes"');
        end;

//********************************************************
//********************************************************
//********************************************************
//********************************************************
//********************************************************
//********************************************************
        end else
        begin
          //Det finnes ikke noe organisasjonsnavn - inkonsistent database!
        
	  //Dette er en ignoreringstillstand.
	  //TODO:  Skrive i en logg at det er en organisasjon som er blitt slettet, men at det er data
	  //         som fortsatt peker p den slettede organisasjonen.

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('Manglende organisasjonsnavn for denne fila.');
{$ENDIF}

        end;

{$IFDEF DEBUG}
//*********  For debugging:  *********
    //Vis info om denne fila
    writeln  ('(orgid, contentid, file, org)', rowbuf[0],', ', rowbuf[2], ', ', rowbuf[1], ', ', cfg.org_name);
    writeln();
{$ENDIF}
        
      end else
      begin
        writeln('Organisasjonsnavn for orgid ', rowbuf[0], ' ikke funnet!');
        writeln();
      end;
    end;
    //Hent en rad til
    rowbuf:=mysql_fetch_row(recbuf);

  end;  //while

  //Frigjr ressurser som ikke lenger behves
  mysql_free_result(recbuf);

{$IFDEF DEBUG}
//*********  For debugging:  *********
  writeln('Kobler ned forbindelsen med mySQL-serveren.');
{$ENDIF}

  //Koble fra mySQL server
  mysql_close(sock);

end;

{****************************************************
 *
 * H O V E D P R O G R A M
 *
 ****************************************************
 *
 * x = ferdig, o = under utvikling
 *
 *  Ufrt:  Plan:
 *  [x]     Hente parametre fra bruker
 *  [x]     Lage "tokenizer" for henting av konfigdata
 *  [x]     Lese konfigurasjonsfil
 *  [x]     "parse" parametrene inn i konfigurasjonsobjekt
 *  [o]     Verifisere de viktigste parametrene i konfigfila
 *  [x]     Koble til databaseserver
 *  [x]     Velg database  bruke
 *  [x]     Hent liste over filer som er ferdig lastet opp og som script ikke er generert for
 ********   For hver fil funnet,
 *  [x]       Hent organisasjonsnavn
 *  [ ]       Generer en tilfeldig tekststreng for  forhindre overskriving av skriptfiler som ikke er prossessert
 *  [ ]       Generer scriptfil for nedlastning av fil
 *  [ ]       Generer *.bat-fil for komprimering av fil
 *  [ ]       Generer scriptfil for opplastning av fil
 *  [ ]       Generer *.xml-fil for komprimering av fil basert p malfil
 ********
 *  [ ]     Endre status i databasen [script er generert]
 *  [ ]     Rydde opp
 *
 ***************************************************}

var
  cfg_file_name:string;  //Filnavn for konfigureringsfil
  cfg:configt;  //Konfigurasjonsparametrene
  result:integer;  //Resultat av i/u-operasjoner
begin

  //Initialiser linjetelleren
  line_count := 0;

  //Generere nytt "random seed" til bruk for  generere tilfeldige filnavn
  randomize();

  // Teste om riktig antall parametere er gitt
  // Tar kun ett parameter: filnavn for konfigureringsfil
  if (paramcount()=1) then

  begin  //Riktig antall parametere, les inn konfigureringsfil
    cfg_file_name := paramstr(1);  //Hent filnavn fra brukerinnmatning

{$IFDEF DEBUG}
//*********  For debugging:  *********
    writeln('Leser konfigureringsfil med navn "' + cfg_file_name + '".');
{$ENDIF}

    //Les inn konfigureringsfil
    result := read_cfg_file(cfg_file_name, cfg);

    if (result=0) then
    begin

{$IFDEF DEBUG}
//*********  For debugging:  *********
      writeln('Fil er lest.');
{$ENDIF}

{$IFDEF DEBUG}
//*********  For debugging:  *********
    write('Kontrollerer konfigurasjonsdata: ');
{$ENDIF}

    //Kontrollere de konfigurasjonsdata som er hentet inn
    verify_config(cfg);

{$IFDEF DEBUG}
//*********  For debugging:  *********
    writeln('Konfigurasjonsdata er ok');
{$ENDIF}

      //Sjekk om det er lastet opp nye filer og generer skript for disse filene
      process_new_files(cfg);

    end else
    begin  //Kunne ikke lese konfigureringsfil
      writeln ('Feil under pning av konfigureringsfil,');
      write ('  feilkode som ble returnert er: ');
      writeln (result);
    end;
  end else	//Feil i antall parametre, forklar brukeren hvordan
  begin
    writeln('sgen v0.1 - script generator for "Envivio 4coder"');
    writeln('Bruk av programmet:');
    writeln('  sgen <config file name>');
  end;
end.
